@@ -22,14 +22,14 @@ HMTOKEN_ADDR = Web3.toChecksumAddress(
     os.getenv("HMTOKEN_ADDR", "0x5FbDB2315678afecb367f032d93F642f64180aa3")
 )
 STAKING_ADDR = Web3.toChecksumAddress(
-    os.getenv("STAKING_ADDR", "0xe7f1725E7734CE288F8367e1Bb143E90bb3F0512")
+    os.getenv("STAKING_ADDR", "0x9fE46736679d2D9a65F0992F2272dE9f3c7fa6e0")
 )
 
 ABIS_FOLDER = os.path.join(os.path.dirname(os.path.dirname(__file__)), "contracts")
 
 # See more details about the eth-kvstore here: https://github.com/hCaptcha/eth-kvstore
 KVSTORE_CONTRACT = Web3.toChecksumAddress(
-    os.getenv("KVSTORE_CONTRACT", "0xCf7Ed3AccA5a467e9e704C703E8D87F634fB0Fc9")
+    os.getenv("KVSTORE_CONTRACT", "0x5FC8d32690cc91D4c39d9d3abcBD16989F875707")
 )
 WEB3_POLL_LATENCY = float(os.getenv("WEB3_POLL_LATENCY", 5))
 WEB3_TIMEOUT = int(os.getenv("WEB3_TIMEOUT", 240))
@@ -335,7 +335,7 @@ def deploy_factory(
     )
 
     txn_func = factory.constructor
-    func_args = [hmtoken_address, staking_address]
+    func_args = []
     txn_info = {
         "gas_payer": gas_payer,
         "gas_payer_priv": gas_payer_priv,
@@ -344,6 +344,12 @@ def deploy_factory(
     }
     txn_receipt = handle_transaction(txn_func, *func_args, **txn_info)
     contract_addr = txn_receipt["contractAddress"]
+
+    factory_contract = get_factory(contract_addr)
+    txn_func = factory_contract.functions.initialize
+    func_args = [staking_address]
+    txn_receipt = handle_transaction(txn_func, *func_args, **txn_info)
+
     return str(contract_addr)
 
 
@@ -377,7 +383,11 @@ def get_staking(staking_addr=HMTOKEN_ADDR, hmt_server_addr: str = None) -> Contr
     return contract
 
 
-def get_pub_key_from_addr(wallet_addr: str, hmt_server_addr: str = None) -> bytes:
+def get_pub_key_from_addr(
+    wallet_addr: str,
+    hmt_server_addr: str = None,
+    kvstore_contract: str = KVSTORE_CONTRACT,
+) -> bytes:
     """
     Given a wallet address, uses the kvstore to pull down the public key for a user
     in the hmt universe, defined by the kvstore key `hmt_pub_key`.  Works with the
@@ -425,8 +435,8 @@ def get_pub_key_from_addr(wallet_addr: str, hmt_server_addr: str = None) -> byte
         "{}/KVStore.sol/KVStore.json".format(ABIS_FOLDER)
     )
 
-    kvstore = w3.eth.contract(address=KVSTORE_CONTRACT, abi=contract_interface["abi"])
-    addr_pub_key = kvstore.functions.get(GAS_PAYER, "hmt_pub_key").call(
+    kvstore = w3.eth.contract(address=kvstore_contract, abi=contract_interface["abi"])
+    addr_pub_key = kvstore.functions.get(wallet_addr, "hmt_pub_key").call(
         {"from": GAS_PAYER}
     )
 
